@@ -198,10 +198,74 @@ namespace eval ::pki {
 namespace eval ::pki::rsa {}
 namespace eval ::pki::x509 {}
 namespace eval ::pki::pkcs {}
+namespace eval ::pki::_util {}
 
 # # ## ### ##### ######## #############
 ## Implementation
 
+proc ::pki::_util::naproc {procname procargs procbody} {
+	set procbody "\n\t::pki::_util::naproc_handleargs [list $procname] [list $procargs] \$args;$procbody"
+	uplevel [list proc $procname args $procbody]
+}
+ 
+proc ::pki::_util::naproc_handleargs {procname procargs calledargs} {
+	set named true
+	if {([llength $calledargs] % 2) != 0} {
+		set named false
+	} else {
+		foreach {arg val} $calledargs {
+			if {[string index $arg 0] ne "-"} {
+				set named false
+
+				break
+			}
+		}
+	}
+	
+	if {!$named} {
+		# Emulate unnamed calling convention
+		for {set i 0} {$i < [llength $procargs]} {incr i} {
+			unset -nocomplain val
+			set arg [lindex $procargs $i]
+			if {[llength $arg] == 2} {
+				set val [lindex $arg 1]
+			}
+			set arg [lindex $arg 0]
+			if {$i < [llength $calledargs]} {
+				set val [lindex $calledargs $i]
+			}
+			if {![info exists val]} {
+				uplevel [list error "wrong # args: should be \"$procname $procargs\""]
+			}
+				uplevel [list set $arg $val]
+		}
+
+		return
+	}
+
+	foreach {arg val} $calledargs {
+		set arg [string range $arg 1 end]
+		set arginfo($arg) $val
+	}
+ 
+	foreach arg $procargs {
+		unset -nocomplain val
+		if {[llength $arg] == 2} {
+			set val [lindex $arg 1]
+		}
+		set arg [lindex $arg 0]
+ 
+		if {[info exists arginfo($arg)]} {
+			set val $arginfo($arg)
+		}
+ 
+		if {![info exists val]} {
+			uplevel [list error "wrong # args: should be \"$procname $procargs\""]
+		}
+		uplevel [list set $arg $val]
+	}
+}
+
 proc ::pki::_dec_to_hex {num} {
 	set retval [format %llx $num]
 	return $retval
@@ -370,7 +434,7 @@ proc ::pki::_unpad_pkcs {data} {
 	return $ret
 }
 
-proc ::pki::rsa::encrypt {mode input keylist} {
+::pki::_util::naproc ::pki::rsa::encrypt {mode input keylist} {
 	switch -- $mode {
 		"pub" {
 			set exponent_ent e
@@ -402,7 +466,7 @@ proc ::pki::rsa::encrypt {mode input keylist} {
 	return $retval
 }
 
-proc ::pki::rsa::decrypt {mode input keylist} {
+::pki::_util::naproc ::pki::rsa::decrypt {mode input keylist} {
 	switch -- $mode {
 		"pub" {
 			set exponent_ent e
@@ -665,27 +729,52 @@ proc ::pki::decrypt args {
 }
 
 # Hash and encrypt with private key
-proc ::pki::sign {input keylist {algo "sha1"}} {
+::pki::_util::naproc ::pki::sign {{input ""} key {algo "sha1"} {channel ""} {file ""}} {
+	if {$channel ne ""} {
+		set extraArgs [list -channel $channel]
+	} elseif {$file ne ""} {
+		set extraArgs [list -file $file]
+	} else {
+		set extraArgs [list $input]
+	}
+
 	switch -- $algo {
 		"md5" {
 			package require md5
 
 			set header "\x30\x20\x30\x0c\x06\x08\x2a\x86\x48\x86\xf7\x0d\x02\x05\x05\x00\x04\x10"
-			set hash [md5::md5 $input]
+			set hash [md5::md5 {*}$extraArgs]
 		}
 		"sha1" {
 			package require sha1
 
 			set header "\x30\x21\x30\x09\x06\x05\x2b\x0e\x03\x02\x1a\x05\x00\x04\x14"
-			set hash [sha1::sha1 -bin $input]
+			set hash [sha1::sha1 -bin {*}$extraArgs]
 		}
 		"sha256" {
 			package require sha256
 
 			set header "\x30\x31\x30\x0d\x06\x09\x60\x86\x48\x01\x65\x03\x04\x02\x01\x05\x00\x04\x20"
-			set hash [sha2::sha256 -bin $input]
+			set hash [sha2::sha256 -bin {*}$extraArgs]
 		}
 		"raw" {
+			if {$channel ne ""} {
+				set input [read $channel]
+			} elseif {$file ne ""} {
+				set channel [open $file]
+				catch {
+					set input [read $channel]
+					set inputOK true
+				} err
+				catch {
+					close $channel
+				}
+
+				if {![info exists inputOK]} {
+					return -code error $err
+				}
+			}
+
 			set header ""
 			set hash $input
 		}
@@ -696,24 +785,32 @@ proc ::pki::sign {input keylist {algo "sha1"}} {
 
 	set plaintext "${header}${hash}"
 
-	array set key $keylist
+	array set keyArray $key
 
-	set padded [::pki::_pad_pkcs $plaintext $key(l) 1]
+	set padded [::pki::_pad_pkcs $plaintext $keyArray(l) 1]
 
-	return [::pki::encrypt -binary -nopad -priv -- $padded $keylist]
+	return [::pki::encrypt -binary -nopad -priv -- $padded $key]
 }
 
 # Verify known-plaintext with signature
-proc ::pki::verify {signedmessage checkmessage keylist {algo default}} {
+::pki::_util::naproc ::pki::verify {signature {message ""} key {algo default} {channel ""} {file ""}} {
+	if {$channel ne ""} {
+		set extraArgs [list -channel $channel]
+	} elseif {$file ne ""} {
+		set extraArgs [list -file $file]
+	} else {
+		set extraArgs [list $message]
+	}
+
 	package require asn
 
 	if {[catch {
-		set plaintext [::pki::decrypt -binary -unpad -pub -- $signedmessage $keylist]
+		set plaintext [::pki::decrypt -binary -unpad -pub -- $signature $key]
 	}]} {
 		return false
 	}
 
-	if {$algo == "default"} {
+	if {$algo eq "default"} {
 		set algoId "unknown"
 		set digest ""
 
@@ -732,20 +829,20 @@ proc ::pki::verify {signedmessage checkmessage keylist {algo default}} {
 
 	switch -- $algoId {
 		"md5" - "md5WithRSAEncryption" {
-			set checkdigest [md5::md5 $checkmessage]
+			set checkdigest [md5::md5 {*}$extraArgs]
 		}
 		"sha1" - "sha1WithRSAEncryption" {
-			set checkdigest [sha1::sha1 -bin $checkmessage]
+			set checkdigest [sha1::sha1 -bin {*}$extraArgs]
 		}
 		"sha256" - "sha256WithRSAEncryption" {
-			set checkdigest [sha2::sha256 -bin $checkmessage]
+			set checkdigest [sha2::sha256 -bin {*}$extraArgs]
 		}
 		default {
 			return -code error "Unknown hashing algorithm: $algoId"
 		}
 	}
 
-	if {$checkdigest != $digest} {
+	if {$checkdigest ne $digest} {
 		return false
 	}
 
@@ -1900,7 +1997,7 @@ proc ::pki::_random args {
 
 	switch -- $dev {
 		"__RAND__" {
-			set ret [expr {int(rand() * 2147483647)}]
+			set ret [expr {int(rand() * 2147483647) & 0xffffff}]
 		}
 		default {
 			set fd [open $dev [list RDONLY BINARY]]
@@ -2127,10 +2224,11 @@ proc ::pki::rsa::generate {bitlength {exponent 0x10001}} {
 	return [array get retkey]
 }
 
+
 ## Initialize parsing routines, which may load additional packages (base64)
 ::pki::_parse_init
 
 # # ## ### ##### ######## #############
 ## Ready
 
-package provide pki 0.9
+package provide pki 0.11
